LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;


entity fifo_buffer_tb is
end fifo_buffer_tb;


architecture my_tb of fifo_buffer_tb is


component fifo 
  generic(
    B: natural:=8; -- number of bits
    W: natural:=4 -- number of address bits
  );
  port(
    clk, reset: in std_logic;
    rd, wr: in std_logic;
    w_data: in std_logic_vector (B-1 downto 0);
    empty, full: out std_logic;
    r_data: out std_logic_vector (B-1 downto 0)
  );
end component;


signal clk_i, reset_i, rd_i, wr_i : std_logic; 
signal w_data_i : std_logic_vector (7 downto 0);

signal empty_o, full_o : std_logic;
signal r_data_o : std_logic_vector (7 downto 0);

 
begin
  
  -- (1) instantiate design under test (DUT);
  DUT: fifo
    generic map(
      B => 8, 
      W => 3 
    )
    port map (
      clk => clk_i, reset => reset_i,
      rd => rd_i, wr => wr_i,
      w_data => w_data_i,
      empty => empty_o, full => full_o,
      r_data => r_data_o 
    );

  
  -- (2) process to generate clock signal;
  clk_gen : process
  begin
    clk_i <= '0'; wait for 1 ps;
    clk_i <= '1'; wait for 1 ps;
  end process clk_gen;
  
  
  -- (3) process to generate desired input stimuli;
  process
  begin
    reset_i <= '1';
    rd_i    <= '0';
    wr_i    <= '0';
    w_data_i<= "00000000";
    wait for 4 ps;

    reset_i <= '0';
    rd_i    <= '0';
    wr_i    <= '1';
    w_data_i<= "00000011";
    wait for 2 ps;
    reset_i <= '0';
    rd_i    <= '0';
    wr_i    <= '1';
    w_data_i<= "00000100";
    wait for 2 ps;
    reset_i <= '0';
    rd_i    <= '0';
    wr_i    <= '1';
    w_data_i<= "00000101";
    wait for 2 ps;
    reset_i <= '0';
    rd_i    <= '0';
    wr_i    <= '1';
    w_data_i<= "00000110";
    wait for 2 ps;
    
    reset_i <= '0';
    rd_i    <= '1';
    wr_i    <= '0';
    w_data_i<= "11111111";
    wait for 2 ps;
    reset_i <= '0';
    rd_i    <= '1';
    wr_i    <= '0';
    w_data_i<= "11111111";
    wait for 2 ps;
    reset_i <= '0';
    rd_i    <= '1';
    wr_i    <= '0';
    w_data_i<= "11111111";
    wait for 2 ps;
    reset_i <= '0';
    rd_i    <= '1';
    wr_i    <= '0';
    w_data_i<= "11111111";
    wait for 2 ps;
    
  end process;
    
end my_tb;
